 EXPORTS
	actx_get_by_node
        command_destroy
        command_get
	ctx_get_by_node
	dictInitIterator
	dictNext
        hiarray_get
	parse_cluster_nodes
	parse_cluster_slots
	redisClusterAppendCommand
	redisClusterAppendCommandArgv
	redisClusterAppendCommandToNode
	redisClusterAppendFormattedCommand
	redisClusterAsyncContextInit
	redisClusterAsyncCommand
	redisClusterAsyncCommandArgv
	redisClusterAsyncCommandArgvToNode
	redisClusterAsyncCommandToNode
	redisClusterAsyncConnect
	redisClusterAsyncConnect2
	redisClusterAsyncDisconnect
	redisClusterAsyncFormattedCommand
	redisClusterAsyncFormattedCommandToNode
	redisClusterAsyncFree
	redisClusterAsyncSetConnectCallback
	redisClusterAsyncSetDisconnectCallback
	redisClusterCommand
	redisClusterCommandArgv
	redisClusterCommandToNode
	redisClusterConnect
	redisClusterConnect2
	redisClusterConnectWithTimeout
	redisClusterContextInit
	redisClusterFormattedCommand
	redisClusterFree
	redisClusterGetNodeByKey
	redisClusterGetReply
	redisClusterGetSlotByKey
	redisClusterInitNodeIterator
	redisClusterNodeNext
	redisClusterReset
	redisClusterSetConnectCallback
	redisClusterSetEventCallback
	redisClusterSetMaxRedirect
	redisClusterSetOptionAddNode
	redisClusterSetOptionAddNodes
	redisClusterSetOptionConnectBlock
	redisClusterSetOptionConnectNonBlock
	redisClusterSetOptionConnectTimeout
	redisClusterSetOptionMaxRetry
	redisClusterSetOptionParseOpenSlots
	redisClusterSetOptionParseSlaves
	redisClusterSetOptionRouteUseSlots
	redisClusterSetOptionTimeout
	redisClusterSetOptionUsername
	redisClusterSetOptionPassword
	redisClusterUpdateSlotmap
	redisClustervAppendCommand
	redisClustervAsyncCommand
	redisClustervCommand
        redis_parse_cmd
